@@ -64,12 +64,15 @@
  */
 
 #include <stdlib.h>
-#include <arpa/inet.h>
 #include <errno.h>
 #include <string.h>
 
 #include <glib.h>
 
+#ifndef G_OS_WIN32
+#  include <arpa/inet.h>
+#endif
+
 #include "pseudotcp.h"
 
 G_DEFINE_TYPE (PseudoTcpSocket, pseudo_tcp_socket, G_TYPE_OBJECT);
@@ -176,17 +179,10 @@ const guint16 PACKET_MAXIMUMS[] = {
 //////////////////////////////////////////////////////////////////////
 // Helper Functions
 //////////////////////////////////////////////////////////////////////
-
-static guint32
-min (guint32 first, guint32 second)
-{
-  return (first < second? first:second);
-}
-static guint32
-max (guint32 first, guint32 second)
-{
-  return (first > second? first:second);
-}
+#ifndef G_OS_WIN32
+#  define min(first, second) ((first) < (second) ? (first) : (second))
+#  define max(first, second) ((first) > (second) ? (first) : (second))
+#endif
 
 static guint32
 bound(guint32 lower, guint32 middle, guint32 upper)
@@ -350,9 +346,20 @@ static void adjustMTU(PseudoTcpSocket *self);
 // The following logging is for detailed (packet-level) pseudotcp analysis only.
 static PseudoTcpDebugLevel debug_level = PSEUDO_TCP_DEBUG_NONE;
 
+#ifndef _MSC_VER
 #define DEBUG(level, fmt, ...) \
   if (debug_level >= level) \
     g_debug ("PseudoTcpSocket %p: " fmt, self, ## __VA_ARGS__)
+#else
+/* HACK ALERT: To avoid Visual Studio compiler error due to the following bug
+ * https://connect.microsoft.com/VisualStudio/feedback/details/604348/-va-args-support-has-an-error
+ * we need to expand the g_debug macro and use g_log directly
+ */
+#define DEBUG(level, fmt, ...) \
+  if (debug_level >= level) \
+    g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "PseudoTcpSocket %p: " fmt, \
+        self, ## __VA_ARGS__)
+#endif
 
 void
 pseudo_tcp_set_debug_level (PseudoTcpDebugLevel level)
@@ -792,16 +799,16 @@ packet(PseudoTcpSocket *self, guint32 seq, guint8 flags,
 
   g_assert(HEADER_SIZE + len <= MAX_PACKET);
 
-  *((uint32_t *) buffer) = htonl(priv->conv);
-  *((uint32_t *) (buffer + 4)) = htonl(seq);
-  *((uint32_t *) (buffer + 8)) = htonl(priv->rcv_nxt);
+  *((guint32 *) buffer) = htonl(priv->conv);
+  *((guint32 *) (buffer + 4)) = htonl(seq);
+  *((guint32 *) (buffer + 8)) = htonl(priv->rcv_nxt);
   buffer[12] = 0;
   buffer[13] = flags;
-  *((uint16_t *) (buffer + 14)) = htons((uint16_t)priv->rcv_wnd);
+  *((guint16 *) (buffer + 14)) = htons((guint16)priv->rcv_wnd);
 
   // Timestamp computations
-  *((uint32_t *) (buffer + 16)) = htonl(now);
-  *((uint32_t *) (buffer + 20)) = htonl(priv->ts_recent);
+  *((guint32 *) (buffer + 16)) = htonl(now);
+  *((guint32 *) (buffer + 20)) = htonl(priv->ts_recent);
   priv->ts_lastack = priv->rcv_nxt;
 
   if (data != NULL)
@@ -839,14 +846,14 @@ parse(PseudoTcpSocket *self, const guint8 * buffer, guint32 size)
   if (size < 12)
     return FALSE;
 
-  seg.conv = ntohl(*(uint32_t *)buffer);
-  seg.seq = ntohl(*(uint32_t *)(buffer + 4));
-  seg.ack = ntohl(*(uint32_t *)(buffer + 8));
+  seg.conv = ntohl(*(guint32 *)buffer);
+  seg.seq = ntohl(*(guint32 *)(buffer + 4));
+  seg.ack = ntohl(*(guint32 *)(buffer + 8));
   seg.flags = buffer[13];
-  seg.wnd = ntohs(*(uint16_t *)(buffer + 14));
+  seg.wnd = ntohs(*(guint16 *)(buffer + 14));
 
-  seg.tsval = ntohl(*(uint32_t *)(buffer + 16));
-  seg.tsecr = ntohl(*(uint32_t *)(buffer + 20));
+  seg.tsval = ntohl(*(guint32 *)(buffer + 16));
+  seg.tsecr = ntohl(*(guint32 *)(buffer + 20));
 
   seg.data = ((gchar *)buffer) + HEADER_SIZE;
   seg.len = size - HEADER_SIZE;
@@ -1141,10 +1148,11 @@ process(PseudoTcpSocket *self, Segment *seg)
           iter = priv->rlist;
         }
       } else {
-        DEBUG (PSEUDO_TCP_DEBUG_NORMAL, "Saving %d bytes (%d -> %d)",
-            seg->len, seg->seq, seg->seq + seg->len);
         GList *iter = NULL;
         RSegment *rseg = g_slice_new0 (RSegment);
+
+        DEBUG (PSEUDO_TCP_DEBUG_NORMAL, "Saving %d bytes (%d -> %d)",
+            seg->len, seg->seq, seg->seq + seg->len);
         rseg->seq = seg->seq;
         rseg->len = seg->len;
         iter = priv->rlist;
